
package monogram.restorer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import util.IOUtil;

/**
 * @author University of Waikato - Te Whare Wānanga o Waikato
 * @version 1.0
 * @since   2014-11-20 
 */
public class TxtRestorer {

    private static final String OUTPUT_CHARSET_ENCODING = "utf-8";
    private Pattern pattern;

    public TxtRestorer() {
        pattern = Pattern.compile("([a-zA-Zā-ūĀ-Ū0-9]+|\\p{Punct})|(.)");
    }

    public void restore(File inputFile, String inputCharsetEncoding, File outputFile, boolean preserveMacrons) {
        MonogramRestorer restorer = new MonogramRestorer(preserveMacrons);
        BufferedReader reader = null;
        BufferedWriter writer = null;
       
        try {
            reader = new BufferedReader(new InputStreamReader(new FileInputStream(inputFile), inputCharsetEncoding));
            writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputFile), OUTPUT_CHARSET_ENCODING));
            String line;
            while ((line = reader.readLine()) != null) {
                final Matcher matcher = pattern.matcher(line);
                while (matcher.find()) {
                    final String token = matcher.group(1);
                    final String nonToken = matcher.group(2);
                    if (token != null) {
                        String restoredToken = restorer.restore(token);
                        writer.write(restoredToken);
                    } else {
                        writer.write(nonToken);
                    }
                }
                writer.newLine();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            IOUtil.closeReader(reader);
            IOUtil.closeWriter(writer);
        }
    }
}
